// The controller contains the logic to interact with the database.

const Book = require('../models/book');

async function getAllBooks(author, title) {
    let searchParams = [];

    if (author) {
        searchParams.push({author: new RegExp(author.toString(), 'i')});
    }
    if (title) {
        searchParams.push({title: new RegExp(title.toString(), 'i')});
    }

    try {
        // Build the query.
        const query = Book.find();

        if (searchParams.length > 0) {
            query.or(searchParams);
        }
        // Execute query and await the result.
        books = await query.exec();
        return {
            success: true,
            data: books,
        };
    } catch (err) {
        return { success: false, message: `Failed to find books: ${err}.`};
    }
}

async function getBookById(id) {
    let book;

    // Find the book.
    try {
        book = await Book.findById(id);
        if (book == null) {
            return { success: false, message: `Cannot find book ID: ${id}` };
        }
    } catch (err) {
        return { success: false, message: err.message };
    }

    return { success: true, data: book }
}

async function addBook(body) {
    const book = new Book(body);

    try {
        const newBook = await book.save();
        return {
            success: true,
            data: newBook,
        };
    } catch (err) {
        return { success: false, message: `Failed to add book: ${err}.` };
    }
}

async function removeBook(id) {
    let book;

    // Find the book and deleted.
    try {
        book = await Book.findByIdAndDelete(id);
        if (book == null) {
            return { success: false, message: 'Cannot find book to delete.' };
        }
    } catch (err) {
        return { success: false, message: err.message };
    }

    return {
        success: true,
        message: `Deleted Book ID:${id}`,
    };
}

// Export controller functions.
module.exports = {
    getAllBooks,
    getBookById,
    addBook,
    removeBook,
}
